public class MainFileManager implements FileManager {
  // THIS IS AN EXAMPLE OF THE SINGLETON PATTERN.

  // AF(this) = A file manager this.manager.
  // RI: manager != null && mostRecent != null

  /** The manager that this manager delegates operations to. */
  private FileManager manager;

  /** The most recent file passed to this manager */
  private String mostRecent;

  /** The singleton instance of this. */
  private static MainFileManager instance;

  /**
   * Constructs a MainFileManager with an empty cache.
   * (Note that this cannot be called by clients!)
   */
  private MainFileManager() {
    this.manager = new SimpleFileManager();
    this.mostRecent = "";

    checkRep();
  }

  /**
   * @return The singleton instance of this.
   */
  public static MainFileManager getInstance() {
    // Return the singleton instance.  Make sure it's initialized!
    if (instance == null) {
      instance = new MainFileManager();
    }

    return instance;
  }

  @Override
  public String get(String file) {
    try {
      checkRep();
      this.mostRecent = file;

      // Delegate the get operation to this.manager.
      return this.manager.get(file);
    } finally {
      checkRep();
    }
  }

  @Override
  public void put(String file, String contents) {
    checkRep();

    this.mostRecent = file;
    // Delegate the put operation to this.manager.
    this.manager.put(file, contents);

    checkRep();
  }

  @Override
  public boolean update(String file) {
    try {
      checkRep();

      this.mostRecent = file;
      // Delegate the update operation to this.manager.
      return this.manager.update(file);
    } finally {
      checkRep();
    }
  }

  /** @return the most recent file passed to any method of this */
  public String getMostRecentFile() {
    return this.mostRecent;
  }

  /**
   * Checks that the representation invariant holds. 
   */
  private void checkRep() {
    assert this.manager != null;
    assert this.mostRecent != null;
  }
}
